@@ -55,6 +55,7 @@ typedef enum {
   PANGO_WEIGHT_ULTRALIGHT = 200,
   PANGO_WEIGHT_LIGHT = 300,
   PANGO_WEIGHT_NORMAL = 400,
+  PANGO_WEIGHT_SEMIBOLD = 600,
   PANGO_WEIGHT_BOLD = 700,
   PANGO_WEIGHT_ULTRABOLD = 800,
   PANGO_WEIGHT_HEAVY = 900
@@ -78,7 +79,8 @@ typedef enum {
   PANGO_FONT_MASK_VARIANT = 1 << 2,
   PANGO_FONT_MASK_WEIGHT  = 1 << 3,
   PANGO_FONT_MASK_STRETCH = 1 << 4,
-  PANGO_FONT_MASK_SIZE    = 1 << 5
+  PANGO_FONT_MASK_SIZE    = 1 << 5,
+  PANGO_FONT_MASK_GRAVITY = 1 << 6
 } PangoFontMask;
 
 /* CSS scale factors (1.2 factor between each size) */
@@ -127,6 +129,12 @@ PangoStretch         pango_font_description_get_stretch       (const PangoFontDe
 void                 pango_font_description_set_size          (PangoFontDescription *desc,
 							       gint                  size);
 gint                 pango_font_description_get_size          (const PangoFontDescription *desc);
+void                 pango_font_description_set_absolute_size (PangoFontDescription *desc,
+							       double                size);
+gboolean             pango_font_description_get_size_is_absolute (const PangoFontDescription *desc);
+void                 pango_font_description_set_gravity       (PangoFontDescription *desc,
+							       PangoGravity          gravity);
+PangoGravity         pango_font_description_get_gravity       (const PangoFontDescription *desc);
 
 PangoFontMask pango_font_description_get_set_fields (const PangoFontDescription *desc);
 void          pango_font_description_unset_fields   (PangoFontDescription       *desc,
@@ -159,6 +167,10 @@ int               pango_font_metrics_get_ascent                  (PangoFontMetri
 int               pango_font_metrics_get_descent                 (PangoFontMetrics *metrics);
 int               pango_font_metrics_get_approximate_char_width  (PangoFontMetrics *metrics);
 int               pango_font_metrics_get_approximate_digit_width (PangoFontMetrics *metrics);
+int               pango_font_metrics_get_underline_position      (PangoFontMetrics *metrics);
+int               pango_font_metrics_get_underline_thickness     (PangoFontMetrics *metrics);
+int               pango_font_metrics_get_strikethrough_position  (PangoFontMetrics *metrics);
+int               pango_font_metrics_get_strikethrough_thickness (PangoFontMetrics *metrics);
 
 #ifdef PANGO_ENABLE_BACKEND
 
@@ -167,11 +179,15 @@ PangoFontMetrics *pango_font_metrics_new (void);
 struct _PangoFontMetrics
 {
   guint ref_count;
-  
+
   int ascent;
   int descent;
   int approximate_char_width;
   int approximate_digit_width;
+  int underline_position;
+  int underline_thickness;
+  int strikethrough_position;
+  int strikethrough_thickness;
 };
 
 #endif /* PANGO_ENABLE_BACKEND */
@@ -213,10 +229,10 @@ struct _PangoFontFamilyClass
   GObjectClass parent_class;
 
   /*< public >*/
-  
+
   void  (*list_faces)      (PangoFontFamily  *family,
-		            PangoFontFace  ***faces,
-		            int              *n_faces);
+			    PangoFontFace  ***faces,
+			    int              *n_faces);
   const char * (*get_name) (PangoFontFamily  *family);
   gboolean (*is_monospace) (PangoFontFamily *family);
 
@@ -242,9 +258,9 @@ GType      pango_font_face_get_type       (void) G_GNUC_CONST;
 
 PangoFontDescription *pango_font_face_describe       (PangoFontFace *face);
 G_CONST_RETURN char  *pango_font_face_get_face_name (PangoFontFace *face);
-void                  pango_font_face_list_sizes     (PangoFontFace  *face, 
-                                                      int           **sizes, 
-                                                      int            *n_sizes);
+void                  pango_font_face_list_sizes     (PangoFontFace  *face,
+						      int           **sizes,
+						      int            *n_sizes);
 
 #ifdef PANGO_ENABLE_BACKEND
 
@@ -264,12 +280,12 @@ struct _PangoFontFaceClass
   GObjectClass parent_class;
 
   /*< public >*/
-  
+
   const char           * (*get_face_name) (PangoFontFace *face);
   PangoFontDescription * (*describe)       (PangoFontFace *face);
-  void                   (*list_sizes)     (PangoFontFace  *face, 
-                                            int           **sizes, 
-                                            int            *n_sizes);
+  void                   (*list_sizes)     (PangoFontFace  *face,
+					    int           **sizes,
+					    int            *n_sizes);
 
   /*< private >*/
 
@@ -292,6 +308,7 @@ struct _PangoFontFaceClass
 GType                 pango_font_get_type          (void) G_GNUC_CONST;
 
 PangoFontDescription *pango_font_describe          (PangoFont        *font);
+PangoFontDescription *pango_font_describe_with_absolute_size (PangoFont        *font);
 PangoCoverage *       pango_font_get_coverage      (PangoFont        *font,
 						    PangoLanguage    *language);
 PangoEngineShape *    pango_font_find_shaper       (PangoFont        *font,
@@ -303,6 +320,7 @@ void                  pango_font_get_glyph_extents (PangoFont        *font,
 						    PangoGlyph        glyph,
 						    PangoRectangle   *ink_rect,
 						    PangoRectangle   *logical_rect);
+PangoFontMap         *pango_font_get_font_map      (PangoFont        *font);
 
 #ifdef PANGO_ENABLE_BACKEND
 
@@ -322,7 +340,7 @@ struct _PangoFontClass
   GObjectClass parent_class;
 
   /*< public >*/
-  
+
   PangoFontDescription *(*describe)           (PangoFont      *font);
   PangoCoverage *       (*get_coverage)       (PangoFont      *font,
 					       PangoLanguage  *lang);
@@ -335,17 +353,30 @@ struct _PangoFontClass
 					       PangoRectangle *logical_rect);
   PangoFontMetrics *    (*get_metrics)        (PangoFont      *font,
 					       PangoLanguage  *language);
+  PangoFontMap *        (*get_font_map)       (PangoFont      *font);
+  PangoFontDescription *(*describe_absolute)  (PangoFont      *font);
   /*< private >*/
 
   /* Padding for future expansion */
   void (*_pango_reserved1) (void);
   void (*_pango_reserved2) (void);
-  void (*_pango_reserved3) (void);
-  void (*_pango_reserved4) (void);
 };
 
+/* used for very rare and miserable situtations that we cannot even
+ * draw a hexbox
+ */
+#define PANGO_UNKNOWN_GLYPH_WIDTH  10
+#define PANGO_UNKNOWN_GLYPH_HEIGHT 14
+
 #endif /* PANGO_ENABLE_BACKEND */
 
+#if defined(PANGO_ENABLE_ENGINE) || defined(PANGO_ENABLE_BACKEND)
+#define PANGO_GLYPH_EMPTY           ((PangoGlyph)0x0FFFFFFF)
+#define PANGO_GLYPH_UNKNOWN_FLAG    ((PangoGlyph)0x10000000)
+#define PANGO_GET_UNKNOWN_GLYPH(wc) ((PangoGlyph)(wc)|PANGO_GLYPH_UNKNOWN_FLAG)
+#endif
+
+
 G_END_DECLS
 
 #endif /* __PANGO_FONT_H__ */
