@@ -46,8 +46,6 @@ typedef struct _GlyphExtentsCacheEntry    GlyphExtentsCacheEntry;
 #define GLYPH_CACHE_NUM_ENTRIES 256 /* should be power of two */
 #define GLYPH_CACHE_MASK (GLYPH_CACHE_NUM_ENTRIES - 1)
 
-#define PANGO_UNITS(Double) ((int)floor((Double) * PANGO_SCALE + 0.5))
-
 /* An entry in the fixed-size cache for the glyph -> ink_rect mapping.
  * The cache is indexed by the lower N bits of the glyph (see
  * GLYPH_CACHE_NUM_ENTRIES).  For scripts with few glyphs,
@@ -72,7 +70,7 @@ struct _PangoCairoFcFont
   PangoGravity gravity;
 
   PangoRectangle font_extents;
-  GlyphExtentsCacheEntry    *glyph_extents_cache;  
+  GlyphExtentsCacheEntry    *glyph_extents_cache;
 };
 
 struct _PangoCairoFcFontClass
@@ -97,14 +95,14 @@ pango_cairo_fc_font_get_font_face (PangoCairoFont *font)
   if (!cffont->font_face)
     {
       cffont->font_face = cairo_ft_font_face_create_for_pattern (fcfont->font_pattern);
-      
+
       /* Unable to create FT2 cairo scaled font.
        * This means out of memory or a cairo/fontconfig/FreeType bug,
        */
       if (!cffont->font_face)
-        return NULL;
+	return NULL;
     }
-  
+
   return cffont->font_face;
 }
 
@@ -120,7 +118,7 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
       font_face = pango_cairo_fc_font_get_font_face (font);
 
       if (!font_face)
-        return NULL;
+	return NULL;
 
       cffont->scaled_font = cairo_scaled_font_create (font_face,
 						      &cffont->font_matrix,
@@ -132,7 +130,7 @@ pango_cairo_fc_font_get_scaled_font (PangoCairoFont *font)
        * or a missing font...
        */
       if (!cffont->scaled_font)
-        return NULL;
+	return NULL;
     }
 
   return cffont->scaled_font;
@@ -183,7 +181,7 @@ pango_cairo_fc_font_finalize (GObject *object)
   if (cffont->glyph_extents_cache)
     {
       g_free (cffont->glyph_extents_cache);
-    }  
+    }
 
   G_OBJECT_CLASS (pango_cairo_fc_font_parent_class)->finalize (object);
 }
@@ -198,15 +196,15 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
   PangoFcFont *fcfont = (PangoFcFont *) (font);
   PangoCairoFcFont *cffont = (PangoCairoFcFont *) (font);
   PangoFcMetricsInfo *info = NULL; /* Quiet gcc */
-  GSList *tmp_list;      
+  GSList *tmp_list;
 
   const char *sample_str = pango_language_get_sample_string (language);
-  
+
   tmp_list = fcfont->metrics_by_lang;
   while (tmp_list)
     {
       info = tmp_list->data;
-      
+
       if (info->sample_str == sample_str)    /* We _don't_ need strcmp */
 	break;
 
@@ -222,9 +220,9 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
 	return pango_font_metrics_new ();
 
       info = g_slice_new0 (PangoFcMetricsInfo);
-      
+
       fcfont->metrics_by_lang = g_slist_prepend (fcfont->metrics_by_lang, info);
-	
+
       info->sample_str = sample_str;
 
       context = pango_fc_font_map_create_context (PANGO_FC_FONT_MAP (fcfont->fontmap));
@@ -247,11 +245,14 @@ pango_cairo_fc_font_get_metrics (PangoFont     *font,
 	    break;
 	  case PANGO_GRAVITY_EAST:
 	  case PANGO_GRAVITY_WEST:
-	    info->metrics->ascent = height / 2;
+	    {
+	      int ascent = height / 2;
+	      if (fcfont->is_hinted)
+	        ascent = PANGO_UNITS_ROUND (ascent);
+	      info->metrics->ascent = ascent;
+	    }
 	}
       shift = (height - info->metrics->ascent) - info->metrics->descent;
-      if (fcfont->is_hinted)
-        shift &= ~(PANGO_SCALE - 1);
       info->metrics->descent += shift;
       info->metrics->underline_position -= shift;
       info->metrics->strikethrough_position -= shift;
@@ -270,7 +271,7 @@ pango_cairo_fc_font_lock_face (PangoFcFont *font)
   cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
   if (G_UNLIKELY (!scaled_font))
     return NULL;
-  
+
   return cairo_ft_scaled_font_lock_face (scaled_font);
 }
 
@@ -281,7 +282,7 @@ pango_cairo_fc_font_unlock_face (PangoFcFont *font)
   cairo_scaled_font_t *scaled_font = pango_cairo_fc_font_get_scaled_font (cfont);
   if (G_UNLIKELY (!scaled_font))
     return;
-  
+
   cairo_ft_scaled_font_unlock_face (scaled_font);
 }
 
@@ -296,20 +297,26 @@ pango_cairo_fc_font_glyph_extents_cache_init (PangoCairoFcFont *cffont)
 
   cffont->font_extents.x = 0;
   cffont->font_extents.width = 0;
-  cffont->font_extents.height = PANGO_UNITS (font_extents.ascent + font_extents.descent);
+  cffont->font_extents.height = pango_units_from_double (font_extents.ascent + font_extents.descent);
   switch (cffont->gravity)
     {
       default:
       case PANGO_GRAVITY_AUTO:
       case PANGO_GRAVITY_SOUTH:
-        cffont->font_extents.y = - PANGO_UNITS (font_extents.ascent);
+	cffont->font_extents.y = - pango_units_from_double (font_extents.ascent);
 	break;
       case PANGO_GRAVITY_NORTH:
-        cffont->font_extents.y = - PANGO_UNITS (font_extents.descent);
+	cffont->font_extents.y = - pango_units_from_double (font_extents.descent);
 	break;
       case PANGO_GRAVITY_EAST:
       case PANGO_GRAVITY_WEST:
-        cffont->font_extents.y = - PANGO_UNITS ((font_extents.ascent + font_extents.descent) * 0.5);
+	{
+	  PangoFcFont *fcfont = (PangoFcFont *) (cffont);
+	  int ascent = pango_units_from_double (font_extents.ascent + font_extents.descent) / 2;
+	  if (fcfont->is_hinted)
+	    ascent = PANGO_UNITS_ROUND (ascent);
+	  cffont->font_extents.y = - ascent;
+	}
     }
 
   cffont->glyph_extents_cache = g_new0 (GlyphExtentsCacheEntry, GLYPH_CACHE_NUM_ENTRIES);
@@ -330,18 +337,18 @@ compute_glyph_extents (PangoCairoFcFont       *cffont,
   cairo_glyph.index = glyph;
   cairo_glyph.x = 0;
   cairo_glyph.y = 0;
-  
+
   cairo_scaled_font_glyph_extents (cffont->scaled_font,
 				   &cairo_glyph, 1, &extents);
-  
+
   entry->glyph = glyph;
-  entry->width = PANGO_UNITS (extents.x_advance);
-  entry->ink_rect.x = PANGO_UNITS (extents.x_bearing);
-  entry->ink_rect.y = PANGO_UNITS (extents.y_bearing);
-  entry->ink_rect.width = PANGO_UNITS (extents.width);
-  entry->ink_rect.height = PANGO_UNITS (extents.height);
+  entry->width = pango_units_from_double (extents.x_advance);
+  entry->ink_rect.x = pango_units_from_double (extents.x_bearing);
+  entry->ink_rect.y = pango_units_from_double (extents.y_bearing);
+  entry->ink_rect.width = pango_units_from_double (extents.width);
+  entry->ink_rect.height = pango_units_from_double (extents.height);
 }
-     
+
 static GlyphExtentsCacheEntry *
 pango_cairo_fc_font_get_glyph_extents_cache_entry (PangoCairoFcFont       *cffont,
 						   PangoGlyph              glyph)
@@ -389,7 +396,7 @@ pango_cairo_fc_font_get_glyph_extents (PangoFont      *font,
 	*logical_rect = cffont->font_extents;
       return;
     }
-  else if (glyph & PANGO_GLYPH_UNKNOWN_FLAG) 
+  else if (glyph & PANGO_GLYPH_UNKNOWN_FLAG)
     {
       _pango_cairo_get_glyph_extents_missing((PangoCairoFont *)font, glyph, ink_rect, logical_rect);
       return;
@@ -430,7 +437,7 @@ pango_cairo_fc_font_class_init (PangoCairoFcFontClass *class)
   PangoFcFontClass *fc_font_class = PANGO_FC_FONT_CLASS (class);
 
   object_class->finalize = pango_cairo_fc_font_finalize;
-  
+
   font_class->get_glyph_extents = pango_cairo_fc_font_get_glyph_extents;
   font_class->get_metrics = pango_cairo_fc_font_get_metrics;
 
@@ -458,13 +465,13 @@ get_font_size (PangoCairoFcFontMap        *cffontmap,
   double size;
 
  /* The reason why we read FC_PIXEL_SIZE here rather than just
-  * using the specified size is to support operations like clamping 
-  * a font to a minimal readable size in fonts.conf. This is pretty weird, 
-  * since it could mean that changing the Cairo CTM doesn't change the 
+  * using the specified size is to support operations like clamping
+  * a font to a minimal readable size in fonts.conf. This is pretty weird,
+  * since it could mean that changing the Cairo CTM doesn't change the
   * font size, but it's just a more radical version of the non-linear
   * font scaling we already have due to hinting and due to bitmap
   * fonts being only available at a few sizes.
-  * 
+  *
   * If honoring FC_PIXEL_SIZE gets in the way of more useful features
   * it should be removed since it only matters in the unusual case
   * of people doing exotic stuff in fonts.conf.
@@ -484,7 +491,7 @@ get_font_size (PangoCairoFcFontMap        *cffontmap,
 
       if (dpi <= 0)
 	dpi = cffontmap->dpi;
-      
+
       return dpi * pango_font_description_get_size (desc) / 72.;
     }
 }
@@ -499,7 +506,7 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
   const PangoMatrix *pango_ctm;
   FcMatrix *fc_matrix;
   double size;
-  
+
   g_return_val_if_fail (PANGO_IS_CAIRO_FC_FONT_MAP (cffontmap), NULL);
   g_return_val_if_fail (pattern != NULL, NULL);
 
@@ -509,21 +516,29 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
 
   cffont->gravity = pango_font_description_get_gravity (desc);
 
+  /* first apply gravity rotation, then pattern matrix, such that
+   * vertical italic text comes out "correct".  we don't do anything
+   * like baseline adjustment etc though.  should be specially
+   * handled when we support italic correction. */
+  cairo_matrix_init_rotate(&cffont->font_matrix,
+			  pango_gravity_to_rotation (cffont->gravity));
+
   if  (FcPatternGetMatrix (pattern,
 			   FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
     {
-      cairo_matrix_init (&cffont->font_matrix,
+      cairo_matrix_t matrix;
+
+      cairo_matrix_init (&matrix,
 			 fc_matrix->xx,
 			 - fc_matrix->yx,
 			 - fc_matrix->xy,
 			 fc_matrix->yy,
 			 0., 0.);
-    }
-  else
-    cairo_matrix_init_identity (&cffont->font_matrix);
 
-  cairo_matrix_rotate(&cffont->font_matrix,
-		      pango_gravity_to_rotation (cffont->gravity));
+      cairo_matrix_multiply (&cffont->font_matrix,
+			     &matrix,
+			     &cffont->font_matrix);
+    }
 
   pango_ctm = pango_context_get_matrix (context);
 
@@ -544,10 +559,10 @@ _pango_cairo_fc_font_new (PangoCairoFcFontMap        *cffontmap,
 
 
   cffont->options = cairo_font_options_copy (_pango_cairo_context_get_merged_font_options (context));
-  
+
   /* fcfont's is_hinted controls metric hinting
    */
-  ((PangoFcFont *)(cffont))->is_hinted = 
+  ((PangoFcFont *)(cffont))->is_hinted =
     (cairo_font_options_get_hint_metrics(cffont->options) != CAIRO_HINT_METRICS_OFF);
 
   return (PangoFcFont *) (cffont);
